/*
 *    behave.js
 *    Copyright  2001 by the IBM Corporation.  All Rights Reserved.
 *
 *    Email to techexpl@us.ibm.com.
 *    Author:  Thomas Ling
 */

var FlagAddMstyle = false;
var FlagMoveTree  = false;
var colorArray  = ['#FF0000', '#0000FF', '#00FF00', '#808080', '#008080',
                   '#F08070', '#FBA123', '#34FBA1', '#FF8000', '#000000'];
var mathAttrs = ['accent', 'alt', 'base', 'bevelled', 'cd', 'closure', 'close', 'columnalign', 'columnspacing', 'display', 
                 'encoding', 'fontfamily', 'fontweight', 'index', 'linethickness', 'mathcolor', 'mathsize', 
                 'maxsize', 'mathvariant', 'nargs', 'notation', 'other', 'open', 'rowspacing', 'type', 'width', null];
var colorIndex = 0;
var fontSize   = 8;
var sibling;

// ----------------------------------------------------------
// Name: IsTokenElem()
//
// Description: 
// This function return true if the input mathml element is
// a token element, otherwise return false
// ----------------------------------------------------------

function IsTokenElem( node )
{
    var nodeName = node.nodeName;

    if ( nodeName == "mi"     || nodeName == "mn"    ||
	     nodeName == "mo"     || nodeName == "mtext" ||
		 nodeName == "mspace" || nodeName == "ms"    ||
		 nodeName == "mglyph" || nodeName == "cn"    ||
		 nodeName == "ci"     || nodeName == "mtd" )
		 return true;

    return false;
}


// ----------------------------------------------------------
// Name: showMathLTree()
//
// Description: 
// This function get the string represntation of the IE tree
// and display it in an alert dialog
// ----------------------------------------------------------

function showMathLTree( mathNode )
{
    var mathelement = getMathElement();
	var IEstring = "This is the MathML IE DOM tree\n"
	IEstring = IEstring + walkNodes( mathelement, "" );
	alert(IEstring);
}


// ----------------------------------------------------------
// Name: walkNodes()
//
// Description: 
// This function walk the IE DOM tree and return a string
// represntation of the mathml part of the tree
// ----------------------------------------------------------

function walkNodes( node, indent )
{
    var str = "";
    indent += "   ";
    
    if ( node == null ) {
        return null;
    }
    // Element nodes
    else if ( node.nodeType == 1 ) {
        str = indent + "<" + node.nodeName.toLowerCase() + ">";

		if (!IsTokenElem(node))
		    str = str + "\n"

        var n = node.childNodes.length;
		
        for ( var i = 0; i < n; i += 1 )
            str = str + walkNodes( node.childNodes.item(i), indent );

        if (!IsTokenElem(node))
		    str = str + indent;

		str =  str + "</" + node.nodeName.toLowerCase() + ">\n";
     }
    // Text nodes
    else if ( node.nodeType == 3 ) {
	    str = str + node.nodeValue
    }
    
    return str;
}


// ----------------------------------------------------------
// Name: getMathElement()
//
// Description: 
// This function find the first math element and returns it
// to the caller
// ----------------------------------------------------------

function getMathElement()
{
    var mathelements = document.getElementsByTagName("math");
    var mathelement = mathelements[0];
	return mathelement;
}


// ----------------------------------------------------------
// Name: changeStyle()
//
// Description: 
// This function add a mstyle element right after the math
// node and change the color and fontsize of a mathml
// expression 
// ----------------------------------------------------------

function changeStyle()
{
	var mathelement = getMathElement();
	colorize( mathelement );
	fontchange( mathelement );
}


// ----------------------------------------------------------
// Name: restoreDOM()
//
// Description: 
// This function remove any style changes or position changes 
// to the mathml element
// ----------------------------------------------------------

function restoreDOM()
{
	var mathelement = getMathElement();

    if (FlagAddMstyle)
	{
	    // Remove the mstyle node from the math element 
		// replace it by it's child

	    var mstyleNode = mathelement.firstChild;
		var mathChild  = mstyleNode.firstChild;
		mathelement.replaceChild(mathChild, mstyleNode);
	    mathelement.validateMml();
		FlagAddMstyle = false;
	}

    if (FlagMoveTree)
	{
	    // make a copy of the tree, remove elements from
		// the original tree and insert the new tree to 
		// IE DOM tree

    	var copyNode = cloneNodes( mathelement );
		removeTextNodes( mathelement, mathelement);
		document.body.removeChild( mathelement );
        document.body.insertBefore(copyNode, sibling);
	    copyNode.validateMml();
		FlagMoveTree = false;
	}
}


// ----------------------------------------------------------
// Name: moveDOMTree()
//
// Description: 
// This function make a copy MathML tree and move it to the
// end of IE tree, then destroy the original tree.
// If it is already moved, then it will reverse the operation. 
// ----------------------------------------------------------

function moveDOMTree()
{
    var mathelement = getMathElement();

    if (!FlagMoveTree)
	{
	    var copyNode = cloneNodes( mathelement );
		removeTextNodes( mathelement, mathelement);
    	sibling = mathelement.nextSibling;
		document.body.removeChild( mathelement );
        document.body.appendChild(copyNode);
	    copyNode.validateMml();
		FlagMoveTree = true;
	}
	else
	{
	   	var copyNode = cloneNodes( mathelement );
		removeTextNodes( mathelement, mathelement);
 		document.body.removeChild( mathelement );
        document.body.insertBefore(copyNode, sibling);
	    copyNode.validateMml();
		FlagMoveTree = false;
	}
}


// ----------------------------------------------------------
// Name: addStyle()
//
// Description: 
// This function create and insert a mstyle element
// between the math element and it's first child.
// ----------------------------------------------------------

function addStyle( mathelement )
{
    if (!FlagAddMstyle)
   	{
        var mstyleNode = document.createElement("mml:mstyle");
        var mathChild  = mathelement.firstChild;
        mathelement.replaceChild(mstyleNode, mathChild);
        mstyleNode.appendChild(mathChild);
	    mathelement.validateMml();
		FlagAddMstyle = true;
	}
}

// ----------------------------------------------------------
// Name: colorize()
//
// Description: 
// This function chnage mstyle's mathcolor to the predfined
// set of color in the colorArray.
// ----------------------------------------------------------

function colorize( mathelement )
{
	addStyle( mathelement );
	if (colorIndex >= 10) colorIndex = 0;
    changeColor( mathelement.firstChild, colorArray[colorIndex++]);
}


// ----------------------------------------------------------
// Name: fontchange()
//
// Description: 
// This function chnage mstyle's mathsize to the predfined
// set of size in the fontSize.
// ----------------------------------------------------------

function fontchange( mathelement )
{
	addStyle( mathelement );
	if (fontSize > 100) fontSize = 8;
    changeSize( mathelement.firstChild, fontSize.toString());  
	fontSize = fontSize + 2;
}


// ----------------------------------------------------------
// Name: changeColor()
//
// Description: 
// This function change the mathcolor attribute of a given node
// ----------------------------------------------------------

function changeColor( node, color)
{
    node.setAttribute("mathcolor", color);
}


// ----------------------------------------------------------
// Name: changeSize()
//
// Description: 
// This function change the mathsize attribute of a given node
// ----------------------------------------------------------

function changeSize(node, size)
{
    node.setAttribute("mathsize", size);
}


// ----------------------------------------------------------
// Name: removeTextNodes()
//
// Description: 
// Remove fine all the textnode and remove them from a
// given node
// ----------------------------------------------------------

function removeTextNodes( mathelem, node )
{
    nodeName = node.nodeName.toLowerCase();

    if (node == null ) {
	    return null;
	}
	else if ( node.nodeType == 1)
	{
        var n = node.childNodes.length;

        for ( var i = 0; i < n; i += 1 )
        {
            var child = removeTextNodes( mathelem, node.childNodes.item(i) );
            
            if (child != null)
            {
                node.removeChild( child );
				mathelem.validateMml();
            }
        }
	}
	else if ( node.nodeType == 3 )
	{
	    return node;
	}

	return null;	    
}


// ----------------------------------------------------------
// Name: cloneNodes()
//
// Description: 
// This function clones all the children of a given node.
// ----------------------------------------------------------

function cloneNodes( node )
{
    var result;

    if ( node == null ) {
        return null;
    }

    // Element nodes
    else if ( node.nodeType == 1 ) {
        var name = "mml:" + node.nodeName.toLowerCase();
        result = document.createElement( name );
        
        var n = node.childNodes.length;

        for ( var i = 0; i < n; i += 1 )
        {
            var child = cloneNodes( node.childNodes.item(i) );
            
            if (child != null)
            {
                result.appendChild( child );
            }
        }

        // copy attributes to the clone element.
        var j = 0;

        while(mathAttrs[j])
	    {
	        var attrib = node.getAttribute(mathAttrs[j]);
 
            if (attrib != null)
 		        result.setAttribute( mathAttrs[j], attrib );
			j++;
		}
    }

    // Text nodes
    else if ( node.nodeType == 3 ) {
        result = document.createTextNode( node.data );
    }

    return result;
}
