/**
 * A simple algebraic calculator using bridge2java and techexplorer.  This code 
 * is based on that found on pp. 466-7 of the book, Core Java, vol. 1 
 *
 * Copyright (c) 2001 by the IBM Corporation. All Rights Reserved.
 *
 * Author:            B. F. Caviness   caviness@us.ibm.com
 *
 * IBM MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IBM SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING OR MODIFYING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import com.ibm.bridge2java.ActiveXCanvas;
import com.ibm.techexplorer.bridge2java.*;      // for access to the TE APIs

class ActiveXCanvas2 extends ActiveXCanvas {
   public Dimension getPreferredSize() {        // Must have these methods for the component to show
      return(new Dimension(230,60));            // in North position when using the border layout mgr
   }
   public Dimension getMinimumSize() {          // The heights and widths used in these two methods
       return(new Dimension(225,60));           // should be relative to the frame size in which
   }                                            // the canvas is contained so the canvas will resize
                                                // properly when the frame is resized, but is not done
}                                               // for simplicity
class CalculatorFrame extends JFrame implements ActionListener {  

   // Instance fields
   private ActiveXCanvas2 activeXCanvas;
   private AxTchExp       axTechExpl;
   private final String   prefix = "\\pagecolor{lime}\\vspace{2pt}$$";   // Used in the display of calculator results
   private String         displayString = "";           // Main part of calculator results to display
   private int            lenLastDisplayComponent = 0;  // Used by the bak button to delete last input
   
   public CalculatorFrame()
   {  setTitle("Symbolic Calculator");
      setSize(230, 250);
      getContentPane().setLayout(new BorderLayout());
      try {
         // Set up the com environment.
         com.ibm.bridge2java.OleEnvironment.Initialize() ;
      }
      catch ( com.ibm.bridge2java.ComException e ) {
          System.out.println( "COM Exception:" ) ;
          System.out.println( Long.toHexString( (e.getHResult()) ) ) ;
          System.out.println( e.getMessage() ) ;
      } catch ( Exception e ) {
          System.out.println( "message: " + e.getMessage() ) ;
      }
      // The canvas must be created and added to the Frame before the 
      // ActiveXObject can get the window handle.
      activeXCanvas = new ActiveXCanvas2();
      getContentPane().add(activeXCanvas,BorderLayout.NORTH);
      setVisible(true);     // Either setVisible() or pack() must occur before creating new 
                            // AxTchExp so that the activeXCanvas isDisplayable().

      // Create an axTechExpl Object and put it in the canvas
      try {
          axTechExpl = new AxTchExp(activeXCanvas.getCanvasHwnd(), 0);
      } catch ( com.ibm.bridge2java.ComException e ) {
          System.out.println( "COM Exception:" ) ;
          System.out.println( Long.toHexString( (e.getHResult()) ) ) ;
          System.out.println( e.getMessage() ) ;
      } catch ( Exception e ) {
          System.out.println( "message: " + e.getMessage() ) ;
      }
      axTechExpl.reloadFromTeXString("\\pagecolor{lime}");  // Set the background color of the canvas
      
      JPanel p = new JPanel();
      p.setLayout(new GridLayout(6, 4));
      addButton(p,"7");
      addButton(p,"8");
      addButton(p,"9");
      addButton(p,"clr");
      String buttons = "456/123*0x";
      for (int i = 0; i < buttons.length(); i++) 
         addButton(p, buttons.substring(i, i + 1));
      addButton(p,"bak");                   // To delete the "last" expr
      addButton(p,"-");
      addButton(p,"exp");
      addButton(p,"log");
      addButton(p,"int");
      addButton(p,"+");                     // Used to compute answer
      addButton(p,"sin");
      addButton(p,"cos");
      addButton(p,"tan");
      addButton(p,"=");                     // Clear the display
      getContentPane().add(p, "Center");
      
      addWindowListener(new WindowAdapter()         // To handle window closing
         {  public void windowClosing(WindowEvent e) {  
               com.ibm.bridge2java.OleEnvironment.UnInitialize();
               System.exit(0);
            }
         } );
   }

   private void addButton(Container c, String s)
   {  JButton b = new JButton(s);
      c.add(b);
      b.addActionListener(this);
   }
  
   public void actionPerformed(ActionEvent evt)
   {  String s = evt.getActionCommand();            // Determine which button was clicked
   
      if (s.equals("=")) {
         evaluate();                                // Carry out some simple symbolic evals
         return;
      }
      else                                          // Add new expr to display str and display
      if (s.equals("exp")||s.equals("log")||s.equals("sin")||s.equals("cos")||s.equals("tan")) {
          displayString += "\\" + s + " x";
          lenLastDisplayComponent = 6;
      }
      else if (s.equals("int")) {
              displayString += "\\int ";
              lenLastDisplayComponent = 5;
           }
      else  if (s.equals("bak")) {
               int newLen = displayString.length() - lenLastDisplayComponent;
               displayString = displayString.substring(0,newLen);
               lenLastDisplayComponent = 0;      // Can only go back one component
           }
      else if (s.equals("clr")) {
            displayString= "";                  // Clear the display
            lenLastDisplayComponent = 0;
           }
      else{
         displayString += s;
         lenLastDisplayComponent = s.length();
      }
      
      axTechExpl.reloadFromTeXString(prefix + displayString +"$$");
   }
   
   // Method for simulating evaluations
   public void evaluate() {  
      if (displayString.equals("x+x+x")||displayString.equals("2*x+x")||displayString.equals("x+2*x")){
        displayString = "3x";
        lenLastDisplayComponent = 2;
      }
      else if (displayString.equals("x+x")) {
            displayString = "2x";
            lenLastDisplayComponent = 2;
           }
      else if (displayString.equals("x-x")) {
            displayString = "0";
            lenLastDisplayComponent = 1;
           }
      else if (displayString.equals("2*x-x")||displayString.equals("x+x-x")||
              displayString.equals("x-x+x")||displayString.equals("-x+2*x")||
              displayString.equals("-x+x+x")||displayString.equals("2x-x")){
            displayString = "x";
            lenLastDisplayComponent = 1;
           }
      else if (displayString.equals("-x-x")) {
            displayString = "-2x";
            lenLastDisplayComponent = 3;
           }
      else if (displayString.equals("x*x*x")||displayString.equals("x^2*x")){
            displayString =  "x^3";
            lenLastDisplayComponent = 3;
           }
      else if (displayString.equals("x*x")){
            displayString =  "x^2";
            lenLastDisplayComponent = 3;
           }
      else if (displayString.equals("x/x")){
            displayString =  "1";
            lenLastDisplayComponent = 1;
           }
      else if (displayString.equals("x/x/x")){
            displayString =  "1/x";
            lenLastDisplayComponent = 3;
           }
      else if (displayString.equals("x*x/x")||displayString.equals("x^2/x")){
            displayString =  "x";
            lenLastDisplayComponent = 1;
           }
      else if (displayString.equals("1/x/x")){
            displayString =  "\\frac{1}{x^2}";
            lenLastDisplayComponent = 13;
           }
      else if (displayString.equals("\\int 1")){
            displayString = "x";
            lenLastDisplayComponent = 1;
           }
      else if (displayString.equals("\\int x")){
            displayString = "\\frac{x^2}{2}";
            lenLastDisplayComponent = 13;
           }
      else if (displayString.equals("\\int 1/x")){
            displayString = "\\log x";
            lenLastDisplayComponent = 6;
           }
      else if (displayString.equals("\\int 1/\\sin x/\\sin x")){
            displayString = "\\tan x";
            lenLastDisplayComponent = 6;
           }
      else if (displayString.equals("\\int \\exp x")){
            displayString = "\\exp x";
            lenLastDisplayComponent = 6;
           }
      else if (displayString.equals("\\int \\log x")){
            displayString = "x\\log x - x";
            lenLastDisplayComponent = 11;
           }
      else if (displayString.equals("\\int \\sin x")){
            displayString = "-\\cos x";
            lenLastDisplayComponent = 7;
           }
      else if (displayString.equals("\\int \\cos x")){
            displayString = "\\sin x";
            lenLastDisplayComponent = 6;
           }
      else if (displayString.equals("\\int \\tan x")){
            displayString = "-\\log\\cos x";
            lenLastDisplayComponent = 11;
           }
           
      axTechExpl.reloadFromTeXString(prefix + displayString + "$$");
   }
}

public class Calculator {  
   
   public static void main(String[] args) {
    JFrame calculator = new CalculatorFrame();
           calculator.setVisible(true);  
   }
}
