/**
 * This testcase demonstrates how to enable the techexplorer ActiveX control 
 * on a canvas thereby enabling the rendering of TeX and MathML markup on the canvas via
 * the methods in the techexplorer Java APIs.  
 *
 * Copyright (c) 2001 by the IBM Corporation. All Rights Reserved.
 *
 * Authors:          Thomas Ling      ling@us.ibm.com  
 *                   B. F. Caviness   caviness@us.ibm.com
 *
 * IBM MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IBM SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING OR MODIFYING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

import java.awt.event.*                         ; // for WindowEvent, WindowListener, ActionEvent, ActionListener
import java.awt.*                               ;
import javax.swing.*                            ; // For JFrame, JTextArea, Box
import com.ibm.bridge2java.ActiveXCanvas        ;
import com.ibm.techexplorer.bridge2java.AxTchExp;

public class TECanvasTest extends JFrame implements ActionListener 
{
    // Instance fields
    private ActiveXCanvas  activeXCanvas;         // A Java canvas capable of holding an ActiveX control
    private AxTchExp       axTechExpl;            // The techexplorer ActiveX control
    private JTextArea      textArea;              // A Java textarea for TeX markup

    // Main method
    public static void main(String[] args)
    {
        TECanvasTest Test = new TECanvasTest();   // Constructor sets up and displays a
                                                  // frame containing the te enabled canvas
                                                  // plus other components that complete
    }                                             // the application.

    // Constructor
    public TECanvasTest()
    {
        super( "techexplorer Canvas Test" );// Sets the title for the frame
        setBounds(200,200,400,200);         // Position the screen at x=y=200 with width of 400 pixels
                                            // and height of 200 pixels
        // Set-up the JFrame
        getContentPane().setLayout(new BorderLayout());

        JButton displayButton = new JButton("Display TeX Markup");
        displayButton.addActionListener(this);
        displayButton.setActionCommand("displayButton");

        textArea = new JTextArea();
		textArea.setBackground(Color.yellow);

        Box b = Box.createHorizontalBox();
        b.add(displayButton);
 		b.add(textArea);
        getContentPane().add("North", b);

        try {
        // Set up the COM environment.
        com.ibm.bridge2java.OleEnvironment.Initialize() ;
        }
        catch ( com.ibm.bridge2java.ComException e )
        {
            System.out.println( "COM Exception:" ) ;
            System.out.println( Long.toHexString( (e.getHResult()) ) ) ;
            System.out.println( e.getMessage() ) ;
        } catch ( Exception e )
        {
            System.out.println( "message: " + e.getMessage() ) ;
        }
        // The canvas must be created and added to the Frame before the 
        // ActiveXObject can get the window handle.
        activeXCanvas = new ActiveXCanvas();
        getContentPane().add(activeXCanvas,BorderLayout.CENTER);
        setVisible(true);                          // Make the window visible

        // Create an axTechExpl Object and put it in the canvas
        try {
            axTechExpl = new AxTchExp(activeXCanvas.getCanvasHwnd(), 0);
        } catch ( com.ibm.bridge2java.ComException e ) {
            System.out.println( "COM Exception:" ) ;
            System.out.println( Long.toHexString( (e.getHResult()) ) ) ;
            System.out.println( e.getMessage() ) ;
        } catch ( Exception e ) {
            System.out.println( "message: " + e.getMessage() ) ;
        }
        
        axTechExpl.reloadFromTeXString("\\pagecolor{lime}");  //Set background of TE canvas
        
    addWindowListener(new WindowAdapter()         // To handle window closing
       {  public void windowClosing(WindowEvent e) {  
             com.ibm.bridge2java.OleEnvironment.UnInitialize();
             System.exit(0);
          }
       } );
       
    }

    // Action Listener
    public void actionPerformed( ActionEvent e ) {
        String action = e.getActionCommand();
        try {
            if (action.equals("displayButton")) {
			    axTechExpl.reloadFromTeXString("\\pagecolor{lime}\\vspace*{0.2in}" + textArea.getText());
            }

        } catch ( com.ibm.bridge2java.ComException ex ) {
            System.out.println( "COM Exception:" ) ;
            System.out.println( Long.toHexString( (ex.getHResult()) ) ) ;
            System.out.println( ex.getMessage() ) ;
        } catch ( Exception ex ) {
            System.out.println( "message: " + ex.getMessage() ) ;
        }
    }
}
