///////////////////////////////////////////////////////////////////////
//
// ConnectEquationEditor.java 
//
///////////////////////////////////////////////////////////////////////
//
// Copyright  1997-99 by the IBM Corporation. All Rights Reserved.
//
// Author:          Angel L. Diaz     aldiaz@us.ibm.com
//
// IBM MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
// THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
// TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
// PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IBM SHALL NOT BE LIABLE FOR
// ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING OR MODIFYING
// THIS SOFTWARE OR ITS DERIVATIVES.
//
///////////////////////////////////////////////////////////////////////

import java.awt.*;
import java.applet.Applet;
import ibm.techexplorer.techexplorer;
import ibm.techexplorer.control.techexplorerControl;
import ibm.techexplorer.axtchexp.AxTchExpRaw;
import ibm.techexplorer.awt.event.MouseListener;
import ibm.techexplorer.awt.event.MouseEvent;
import ibm.techexplorer.dom.*;
import ibm.techexplorer.util.*;
import org.w3c.dom.*;


public class ConnectEquationEditor extends java.applet.Applet implements MouseListener
{  
	private techexplorer techexplorer = null;
	private Node currentNode = null;
	private boolean debug = true;
	private DhObject dHTML = null;
			
	public void setPlugin( techexplorer obj )
	{
		System.out.println( "setPlugin" );

		if ( obj instanceof techexplorer )
			techexplorer = obj;

        dHTML = new DhObject((Applet) this);
		initMathematica();
		initListeners();
	}

	public void setControl( AxTchExpRaw obj, Object dhtmlobj )
	{
		System.out.println( "setControl" );

		if ( obj instanceof AxTchExpRaw )
			techexplorer = new techexplorerControl( obj );

        dHTML = new DhObject(dhtmlobj);
		initMathematica();
		initListeners();
	}

	public void setEquationFromLaTeX( String expression )
	{
		if (debug) System.out.println( "setEquationFromLaTeX: " + expression );
		if (debug) System.out.println( "setEquationFromLaTeX: " + removeInlineMath( expression ) );

		if ( currentNode != null )
		{
			TEDocument teDocument = (TEDocument) techexplorer.getDocumentNode();

			Node latexNode = teDocument.createFromTexString( "$" + removeInlineMath( expression ) + "$" );
			
			if (debug) System.out.println( "setEquationFromLaTeX: " + teDocument.getTexString( latexNode ) );

			Node parent = currentNode.getParentNode();
			parent.replaceChild( latexNode, currentNode );

			currentNode = null;

			teDocument.recomposeAndRedraw();   
		}
		else
			System.out.println( "setEquationFromLaTeX: current node is null" );
	}

	private String removeInlineMath( String string )
	{
		StringBuffer stringBuffer = new StringBuffer();
		char[] stringArray = string.toCharArray();

		for( int i=0; i<string.length(); i++ )
		{
			if ( stringArray[i] == '$' )
				continue; 
			else
				stringBuffer.append( stringArray[i] ); 	
		}

		return stringBuffer.toString();
	}

	private Node cloneNode( TEDocument doc, Node node )
	{
		Node result = null;

		if ( node == null ) 
		{
			return null;
		}

		// Element nodes
		else if ( node.getNodeType() == node.ELEMENT_NODE  ) 
		{
			String name = node.getNodeName();
			if ( debug) System.out.println( "cloneNode: " + name );

			result = doc.createElement( name );

			int n = node.getChildNodes().getLength();

			for ( int i = 0; i < n; i += 1 )
				result.appendChild( cloneNode( doc, node.getChildNodes().item(i) ) );

			n = node.getAttributes().getLength();

			for ( int i = 0; i < n; i += 1 )
				((Element)result).setAttribute( node.getAttributes().item(i).getNodeName(), node.getAttributes().item(i).getNodeValue() );		
		}

		// Text nodes
		else if ( node.getNodeType() == node.TEXT_NODE ) 
		{
			if ( debug) System.out.println( "cloneNode: text " + ((Text)node).getData() );

			result = doc.createTextNode( ((Text)node).getData() );
		}

		return result;
	}

	private void initMathematica()
	{
		if ( techexplorer != null )
		{
			/*
			int nRval = techexplorer.addInInitialize( "application/x-mathlink-mat-tex" );
			switch( nRval ) 
			{
				case techexplorer.ADDIN_SUCCESS:
				System.out.println( "setPlugin: Loaded" );
				break;
				case techexplorer.ADDIN_ERROR_AVAILABLE:
				System.out.println( "setPlugin: add-in not available" );
				break;
				case techexplorer.ADDIN_ERROR_LOAD:
				System.out.println( "setPlugin: can not load add-in dll" );
				break;
				default:
				System.out.println( "setPlugin: Not Loaded" );
				break;
			}
			*/
		}
	}

	private void initListeners()
	{
		if ( techexplorer != null )
			techexplorer.addMouseListener( (MouseListener)this );
	}

	public void init() 
	{
		System.out.println( "ConnectEquationEditor: init" );
	}

	public void start() 
	{     
		System.out.println( "ConnectEquationEditor: start" );
	}

	public void stop() 
	{
		System.out.println( "ConnectEquationEditor: stop" );
	}

	public void mouseClicked( MouseEvent e ) 
	{ 
	}

	public void mousePressed( MouseEvent e ) 
	{
	}
    
	public void mouseReleased( MouseEvent e ) 
	{
		System.out.println( "mouseReleased" );

		selectNodeList( e.getNodeSelectionList() );
	}

	public void mouseEntered( MouseEvent e ) 
	{
	}

	public void mouseExited( MouseEvent e ) 
	{
	}

	private void selectNodeList( NodeList nodeList )
	{
		if ( nodeList != null  )  		
		{

			if (debug) 
			{
				for( int i=0; i<nodeList.getLength(); i++ )
				{
					Node node = nodeList.item(i);

					System.out.println( "selectNodeList: node name: " + node.getNodeName() );
					System.out.println( "selectNodeList: node value: " + node.getNodeValue() );
				}
			}

  			/* get first inline-math node */
			currentNode = null;
			

			for( int i=0; i<nodeList.getLength(); i++ )
			{

  				currentNode = getInlineMath( nodeList.item(i) );

				if ( currentNode != null )
				{
					TEDocument teDocument = (TEDocument) techexplorer.getDocumentNode();
					String latexString = teDocument.getTexString( currentNode );

					System.out.println( latexString );
					setSelectionAttributeForChildren( currentNode, "true" );
					teDocument.recomposeAndRedraw();

                    if (dHTML != null)
                    {
                        Object args[] = new Object[1];
                        args[0] = new String(latexString);
                        dHTML.call("setEquation", args);
                    }

					break;
				}
 			}		
		}
		else
			System.out.println( "selectNodeList: null node list" );	
	}

	private Node getInlineMath( Node node )
	{
		Node inlineMathNode = null;
		Node target = node;

		while( target != null )
		{
			if ( target.getNodeName().compareTo( "inline-math" ) == 0  )
			{
				inlineMathNode = target;
				break;
			}
			target = target.getParentNode();
		}

		return inlineMathNode;
	}

	private void setSelectionAttributeForChildren( Node node, String value )
	{
		setSelectionAttribute( node, value );
		if ( node.hasChildNodes() )
		{
			NodeList nodeList = node.getChildNodes();

			for( int i=0; i<nodeList.getLength(); i++)
				setSelectionAttributeForChildren( nodeList.item( i ), value );	
		}
	}
	  
	private void setSelectionAttribute( Node node, String value )
	{
		NamedNodeMap nodeMap = node.getAttributes();
		if ( nodeMap != null )
		{
			int nAttributes = nodeMap.getLength();

			for(int i=0; i<nAttributes; i++ )
			{
				Attr attr = (Attr)nodeMap.item(i);

				if ( attr.getName().compareTo( "is-selected" ) == 0 )
					attr.setValue( value );
			}
		}
	}

	private String normalizeString( String string )
	{
		StringBuffer stringBuffer = new StringBuffer();
		char[] stringArray = string.toCharArray();

		for( int i=0; i<string.length(); i++ )
		{
			if ( stringArray[i] == '\\' )
				stringBuffer.append( "\\\\" );
			else
				stringBuffer.append( stringArray[i] ); 	
		}

		return stringBuffer.toString();
	}

	public void evaluate( String value ) 
	{
	}
}
