///////////////////////////////////////////////////////////////////////
//
// ExpressionSelector.java : 
//
///////////////////////////////////////////////////////////////////////
//
// Copyright  1997-99 by the IBM Corporation. All Rights Reserved.
//
// Author:          Angel L. Diaz     aldiaz@us.ibm.com
//                  Robert S. Sutor   sutor@us.ibm.com
//
// IBM MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
// THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
// TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
// PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IBM SHALL NOT BE LIABLE FOR
// ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING OR MODIFYING
// THIS SOFTWARE OR ITS DERIVATIVES.
//
///////////////////////////////////////////////////////////////////////

import java.awt.*;
import java.applet.Applet;

import ibm.techexplorer.techexplorer;
import ibm.techexplorer.control.techexplorerControl;
import ibm.techexplorer.axtchexp.AxTchExpRaw;
import ibm.techexplorer.event.InstanceEvent;
import ibm.techexplorer.event.InstanceListener;
import ibm.techexplorer.awt.AWTEvent;
import ibm.techexplorer.awt.event.KeyListener;
import ibm.techexplorer.awt.event.KeyEvent;
import ibm.techexplorer.awt.event.FocusListener;
import ibm.techexplorer.awt.event.FocusEvent;
import ibm.techexplorer.awt.event.MouseEvent;
import ibm.techexplorer.awt.event.MouseListener;
import ibm.techexplorer.awt.event.MouseMotionListener;
import ibm.techexplorer.dom.*;
import org.w3c.dom.*;

public class ExpressionSelector extends java.applet.Applet implements MouseListener, MouseMotionListener {
	private static final boolean debug              = false;

	private static final int   SELECT_NODELIST      = 1;
	private static final int   PATH_NODELIST        = 2;
	private static final int   MOTION_NODELIST      = 3;
	private int                currentOperation     = MOTION_NODELIST;
	
	private techexplorer       techexplorerObj1     = null;
	private techexplorer       techexplorerObj2     = null;
	private TEDocument         techexplorerDoc1     = null;
	private TEDocument         techexplorerDoc2     = null;

	private Button             nodePathList         = null;
	private Button             selectionNodeList    = null;
	private Button             motionNodeList       = null;

    private Button             insertLaTeX          = null;
    
    private TextField          markupTextField      = null;
    
	private Node               currentNode          = null;
	private Node               inlineMathDoc2       = null;
	private Node               inlineMathDoc1       = null;

    public void setPlugins( techexplorer plugin1, techexplorer plugin2 )
    {
        if ( plugin1 instanceof techexplorer )
            techexplorerObj1 = plugin1;

        if ( plugin2 instanceof techexplorer )
            techexplorerObj2 = plugin2;

		initEvents();
    }

    public void setControls( AxTchExpRaw control1, AxTchExpRaw control2 )
    {
        if ( control1 instanceof AxTchExpRaw )
            techexplorerObj1 = new techexplorerControl( control1 );

        if ( control2 instanceof AxTchExpRaw )
            techexplorerObj2 = new techexplorerControl( control2 );

		initEvents();
    }

	public void init() 
	{
        
		nodePathList = new Button("Node Path");
		selectionNodeList = new Button("Selection Path");
		motionNodeList = new Button("Motion Path");

        Panel expressionButtonPanel = new Panel();
		expressionButtonPanel.setLayout(new GridLayout(1, 3));
		expressionButtonPanel.add(nodePathList);
		expressionButtonPanel.add(selectionNodeList);
		expressionButtonPanel.add(motionNodeList);
        
        markupTextField = new TextField();

        insertLaTeX = new Button( "Insert LaTeX" );
        
        Panel insertButtonPanel = new Panel();
		insertButtonPanel.setLayout(new GridLayout(1, 1));
        insertButtonPanel.add( insertLaTeX );
                
		setLayout(new GridLayout(3, 1));                
        add( expressionButtonPanel );
        add( markupTextField );
        add( insertButtonPanel );
	}
  
	public void start() 
	{
		if (debug) System.out.println("ExpressionSelector: start()");
	}

    public void destroy()
	{
	    // clean up non-java object
	    techexplorerObj1.gc();
		techexplorerObj2.gc();
	}

	public void initEvents()
	{
				
		if ( techexplorerObj1 == null ) 
		{
			System.out.println("ExpressionSelector: start(): null techexplorerPlugin1");
		}
		else 
		{
			techexplorerObj1.addMouseListener( (MouseListener)this );
			techexplorerObj1.addMouseMotionListener( (MouseMotionListener)this );
			techexplorerObj1.disableEvents( AWTEvent.MOUSE_EVENT_MASK );
			techexplorerDoc1 = (TEDocument) techexplorerObj1.getDocumentNode();
		}

		if ( techexplorerObj2 == null ) 
		{
			System.out.println("ExpressionSelector: start(): null techexplorerPlugin2");
		}
		else 
		{
			techexplorerObj2.disableEvents( AWTEvent.MOUSE_EVENT_MASK );
			techexplorerDoc2 = (TEDocument) techexplorerObj2.getDocumentNode();
			inlineMathDoc2 = findFirstNodeFromName( techexplorerDoc2, "inline-math" );	  
		}
	}

	public void stop() 
	{
		if (debug) System.out.println("EventMonitor: stop()");
	}

	public boolean action(Event evt, Object arg) 
	{
		setSelectionAttributeForChildren( techexplorerDoc1, "false" );
		techexplorerDoc1.recomposeAndRedraw();
		 
		if ( evt.target == nodePathList )
		{ 
			if (debug) System.out.println("Node Path");
			techexplorerObj1.disableEvents( AWTEvent.MOUSE_EVENT_MASK );
			techexplorerObj1.removeMouseMotionListener( (MouseMotionListener)this );
			currentOperation = PATH_NODELIST;
		}
		else if ( evt.target == motionNodeList )
		{
			if (debug) System.out.println("Motion Path");
			techexplorerObj1.addMouseMotionListener( (MouseMotionListener)this );
			techexplorerObj1.disableEvents( AWTEvent.MOUSE_EVENT_MASK );
			currentOperation = MOTION_NODELIST;
		}
		else if ( evt.target == selectionNodeList ) 
		{
			if (debug) System.out.println("Selection Path");
			techexplorerObj1.enableEvents( AWTEvent.MOUSE_EVENT_MASK );
			techexplorerObj1.removeMouseMotionListener( (MouseMotionListener)this );
			currentOperation = SELECT_NODELIST;
		}
        else if ( evt.target == insertLaTeX )
        {
			if (debug) System.out.println("Insert LaTeX");
            String markup = markupTextField.getText();
            
            if ( markup != null )
            {
                Node node = techexplorerDoc1.createFromTexString( markup );
                
                insertNode( node );
            }
        }
                
		return true;	
	}

	public void mouseClicked( MouseEvent e ) 
	{ 
	}

	public void mousePressed( MouseEvent e ) 
	{
	}

    public void insertNode( Node node )
    {
        if ( node != null )
        {
  			inlineMathDoc1 = findFirstNodeFromName( techexplorerDoc1, "inline-math" );
			removeAllChildren( inlineMathDoc1 );

 			inlineMathDoc2 = findFirstNodeFromName( techexplorerDoc2, "inline-math" );
			removeAllChildren( inlineMathDoc2 );
            
            inlineMathDoc1.appendChild( node );
        }
    }
    
	public void selectNodeList( NodeList nodeList )
	{
		if ( nodeList != null )
		{
			inlineMathDoc2 = findFirstNodeFromName( techexplorerDoc2, "inline-math" );
			removeAllChildren( inlineMathDoc2 );

			int nLength = nodeList.getLength();

			for( int i=0; i<nLength; i++ )
			{
				Node node = nodeList.item(i);

				if (debug) System.out.println( "selectNodeList: node name: " + node.getNodeName() );
				if (debug) System.out.println( "selectNodeList: node value: " + node.getNodeValue() );

				inlineMathDoc2.appendChild( cloneNodeAllChildren( node, techexplorerDoc2 ) );
			}
			techexplorerDoc2.recomposeAndRedraw();
		}
		else
			System.out.println( "selectNodeList: null node list" );
	}
   
	public void pathNodeList( NodeList nodeList )
	{
		if ( nodeList != null )
		{
			Node node = nodeList.item(nodeList.getLength() - 1); 
		
			if (debug) System.out.println( "pathNodeList: node name: " + node.getNodeName() );
			if (debug) System.out.println( "pathNodeList: node value: " + node.getNodeValue() );

			inlineMathDoc2 = findFirstNodeFromName( techexplorerDoc2, "inline-math" );
			removeAllChildren( inlineMathDoc2 );

			inlineMathDoc2.appendChild( cloneNodeAllChildren( node, techexplorerDoc2 ) );
			techexplorerDoc2.recomposeAndRedraw();

			if ( debug) System.out.println( getDOMString( techexplorerDoc2, 0 ) );

		}
		else
			System.out.println( "pathNodeList: null node list" );
			
	}

	public void motionNodeList( NodeList nodeList )
	{
		if ( nodeList != null ) {
			if ( currentNode != nodeList.item(nodeList.getLength()-1) )
			{ 
				if ( currentNode != null )
					setSelectionAttribute( currentNode, "false" );

				currentNode = nodeList.item(nodeList.getLength()-1);
				setSelectionAttribute( currentNode, "true" );

				inlineMathDoc2 = findFirstNodeFromName( techexplorerDoc2, "inline-math" );
				removeAllChildren( inlineMathDoc2 );

				inlineMathDoc2.appendChild( cloneNodeAllChildren( currentNode, techexplorerDoc2 ) );
			
				techexplorerDoc2.recomposeAndRedraw();
				techexplorerDoc1.recomposeAndRedrawNode( currentNode );
			}
		}
		else
			System.out.println( "motionNodeList: null node list" );
	}

	public void mouseReleased( MouseEvent e ) 
	{
		System.out.println( "mouseRelease" );
		switch ( currentOperation )
		{
			case SELECT_NODELIST:
				selectNodeList( e.getNodeSelectionList() );
			break;
			case PATH_NODELIST:
				pathNodeList( e.getNodePathList() ); 
			break;
		}  
	}

	public void mouseEntered( MouseEvent e ) 
	{ 
	}

	public void mouseExited( MouseEvent e )  
	{ 
	}

	public void mouseDragged( MouseEvent e ) 
	{ 
	}

	public void mouseMoved( MouseEvent e ) 
	{
		if ( currentOperation == MOTION_NODELIST )
			motionNodeList( e.getNodePathList() ); 
	}
 
	public void setSelectionAttributeForChildren( Node node, String value )
	{
		setSelectionAttribute( node, value );
		if ( node.hasChildNodes() )
		{
			NodeList nodeList = node.getChildNodes();

			for( int i=0; i<nodeList.getLength(); i++)
				setSelectionAttributeForChildren( nodeList.item( i ), value );	
		}
	}
	  
	public void setSelectionAttribute( Node node, String value )
	{
		NamedNodeMap nodeMap = node.getAttributes();
		if ( nodeMap != null )
		{
			int nAttributes = nodeMap.getLength();

			for(int i=0; i<nAttributes; i++ )
			{
				Attr attr = (Attr)nodeMap.item(i);

				if ( attr.getName().compareTo( "is-selected" ) == 0 )
					attr.setValue( value );
			}
		}
	}

	private Node findFirstNodeFromName( Node node, String name )
	{
		Node targetNode = null;
		 
		if (debug) System.out.println( "findFirstNodeFromName: current node name: " + node.getNodeName() );
		if ( node.getNodeType() == Node.TEXT_NODE )
		   if (debug) System.out.println( "findFirstNodeFromName: current node value: " + node.getNodeValue() );
			
	  	if ( node.getNodeName().compareTo( name ) != 0 )
		{
			if ( node.hasChildNodes() )
			{
				NodeList nodeList = node.getChildNodes();
				if (debug) System.out.println( "findFirstNodeFromName: current node number of children: " + nodeList.getLength() ); 
				for( int i=0; i<nodeList.getLength(); i++ )
				{
					targetNode = findFirstNodeFromName( nodeList.item( i ), name );
					if ( targetNode != null )
						break;  
		  		}
			}
			else
				if (debug)  System.out.println( "findFirstNodeFromName: current node no children" );

	    }
		else
			targetNode = node;

		return targetNode;
	}

	public void removeAllChildren( Node node )
	{
		if ( node.hasChildNodes() )
		{
			NodeList nodeList = node.getChildNodes();

			for( int i=0; i<nodeList.getLength(); i++ )
			{
				if (debug) System.out.println( "removeAllChildren: bye bye to: " + nodeList.item( i ).getNodeName() );
				node.removeChild( nodeList.item( i ) );
			}
		}
		else
			System.out.println( "removeAllChildren: no children" );	
	}

	public String getDOMString(Node node, int nLevel) 
	{
		StringBuffer stringBuffer = new StringBuffer(); 
	    String nodeName   = null;
	    String nodeValue  = null;
		NodeList nodeList = null;
		int   size        = 0;

    	nodeName  =  node.getNodeName();
    	nodeValue =  node.getNodeValue();

		switch( node.getNodeType() )
		{ 
			case Node.TEXT_NODE:
				String trimValue = nodeValue.trim();

				if ( trimValue.compareTo( "" ) != 0 )
					stringBuffer.append( indentation( nLevel ) + nodeValue.trim() + "\n");
			break;
			case Node.DOCUMENT_NODE:
			break; 
			default:
		    	stringBuffer.append( indentation( nLevel ) + "<" + nodeName + ">\n" );
			break;
		}

	    if ( node.hasChildNodes() ) {
	    	nodeList = node.getChildNodes();
			size  = nodeList.getLength();

			for (int i = 0; i < size; i++) {
				stringBuffer.append( getDOMString(  nodeList.item(i), nLevel + 1) );
			}
		}

		switch( node.getNodeType() )
		{ 
			case Node.TEXT_NODE:
			break;
			case Node.DOCUMENT_NODE:
			break; 
			default:
		    	stringBuffer.append( indentation( nLevel ) + "</" + nodeName + ">\n" );
			break;
		}    

		return stringBuffer.toString();
	}

	public String indentation( int nLevel )
	{
		String sIndentation = "";

		for( int i=0; i<nLevel; i++ )
			sIndentation += "  ";

		return sIndentation;
	}

	private Node cloneNode(Node node, Document document)
	{
		Node newNode = null;

		try {
			switch( node.getNodeType() )
			{
				case Node.ELEMENT_NODE:
					newNode = document.createElement( node.getNodeName() );
					if ( node.getNodeName().compareTo( "control-sequence" ) == 0 )
						((Element)newNode).setAttribute( "control-sequence", ((Element)node).getAttribute( "control-sequence" ) );
				break;
				case Node.TEXT_NODE:
					newNode = document.createTextNode( node.getNodeValue() );
				break;
				default:
					System.out.println( "cloneNode: ooooooooooooooops " + node.getNodeType() );
				break;
			}
		} catch( Exception e) { System.out.println( "cloneNode: " + e ); }

		return newNode;
	}

	private Node cloneNodeAllChildren(Node sourceNode, Document document )
	{
		
		Node targetNode = cloneNode( sourceNode, document );

		try
		{
			if (debug) System.out.println( "cloneNodeAllChildren: source node:  " + sourceNode.getNodeName() );
			if (debug) System.out.println( "cloneNodeAllChildren: target node:  " + targetNode.getNodeName() );
			
			if ( sourceNode.hasChildNodes() )
			{
				NodeList nodeList = sourceNode.getChildNodes();

				for(int i=0; i<nodeList.getLength(); i++)
				{
					Node nodeCopy = cloneNodeAllChildren( nodeList.item( i ), document );

					if ( debug )
						System.out.println( "cloneNodeAllChildren: appending " + 
						nodeCopy.getNodeName() + " to " + targetNode.getNodeName() ); 
					
						/*
						 * we really need to document what works and does not for each node
						 */

						if ( targetNode.hasChildNodes() &&
							 targetNode.getNodeName().compareTo( "top" ) != 0 &&
							 targetNode.getNodeName().compareTo( "bottom" ) != 0 &&
							 targetNode.getNodeName().compareTo( "nucleus" ) != 0 &&
							 targetNode.getNodeName().compareTo( "superscript" ) != 0 &&
							 targetNode.getNodeName().compareTo( "subscript" ) != 0
						   )
  							targetNode.replaceChild( nodeCopy, targetNode.getChildNodes().item( i ) );
						else
							targetNode.appendChild( nodeCopy );
				}
			}
		} catch( Exception e) { System.out.println( "cloneNodeAllChildren: " + e ); }

		return targetNode;
	}

}
