//////////////////////////////////////////////////////////////////////////////
//
// MenuTest.java : MenuTest applet
//
//////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2000 by the IBM Corporation. All Rights Reserved.
//
// Author:              Samuel S. Dooley        dooley@watson.ibm.com
//
// IBM MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
// THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
// TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
// PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IBM SHALL NOT BE LIABLE FOR
// ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING OR MODIFYING
// THIS SOFTWARE OR ITS DERIVATIVES.
//
//////////////////////////////////////////////////////////////////////////////

import java.applet.Applet;

import org.w3c.dom.*;

import ibm.techexplorer.dom.TEDocument;
import ibm.techexplorer.techexplorer;

import ibm.techexplorer.control.techexplorerControl;
import ibm.techexplorer.axtchexp.AxTchExpRaw;

//////////////////////////////////////////////////////////////////////////////

public class MenuTest extends Applet
{
    /*
     * Attributes
     */

    /*
     * techexplorer
     */

    private techexplorer techexplorer = null;

    public void setPlugin( techexplorer obj )
    {
        if ( obj instanceof techexplorer )
            techexplorer = obj;
    }

    public void setControl( AxTchExpRaw obj )
    {
        if ( obj instanceof AxTchExpRaw )
            techexplorer = new techexplorerControl( obj );
    }

    /*
     * document
     */

    private Document ownerDocument = null;

    public Document getDocument()
    {
        if ( ownerDocument != null )
            return ownerDocument;

        if ( techexplorer != null )
            ownerDocument = techexplorer.getDocumentNode();

        return ownerDocument;
    }

    private Node parent = null;
    private Node bcs = null;
    private Node ecs = null;

    /*
     * warmup
     */

    public void warmup()
    {
        ownerDocument = null;

        Document doc = getDocument();
        if ( doc == null ) {
            System.out.println( "warmup:  null document" );
            return;
        }

        Node elt = doc.getDocumentElement();
        if ( ! isNodeName( elt, "document" ) ) {
            System.out.println( "warmup:  invalid document element" );
            return;
        }

        Node node = elt.getFirstChild();
        if ( ! isNodeName( node, "vertical-list" ) ) {
            System.out.println( "warmup:  invalid vertical list" );
            return;
        }

        while ( node.getFirstChild() == null ) {
        }
    }

    /*
     * begin
     */

    public void begin()
    {
        try {
            warmup();
            createDisplay();
            redraw();
        }
        catch ( Exception e ) {
            e.printStackTrace();
        }
    }

    /*
     * createItem
     */

    public void createItem( String arg )
    {
        try {
            System.out.println( "createItem" );
            Document doc = getDocument();

            Node item = doc.createElement( "item" );
            item.appendChild( createControlSequence( "bullet" ) );

            Node para = doc.createElement( "paragraph" );
            para.appendChild( item );
            para.appendChild( createControlSequence( "_SECRETSTRUT" ) );
            para.appendChild( createItemMenu( arg ) );

            parent.insertBefore( para, ecs );
            redraw();
        }
        catch ( Exception e ) {
            e.printStackTrace();
        }
    }

    /*
     * deleteStep
     */

    public void deleteItem( String arg )
    {
        try {
            System.out.println( "deleteItem" );
            Node para = findControlSequence( arg );
            while ( para != null ) {
                para = para.getParentNode();
                if ( para.getNodeName().equals( "paragraph" ) )
                    break;
            }

            if ( para == null )
                return;

            para.getParentNode().removeChild( para );
            redraw();
        }
        catch ( Exception e ) {
            e.printStackTrace();
        }
    }

    /*
     * createDisplay
     */

    private void createDisplay()
    {
        // \\begin{itemize}
        // \\end{itemize}

        Node node = findElementByTagName( "centerline" );
        bcs = createControlSequence( "beginitemize" );
        ecs = createControlSequence( "enditemize" );

        parent = node.getParentNode();
        parent.insertBefore( bcs, node );
        parent.insertBefore( ecs, node );
        parent.removeChild( node );
    }

    /*
     * createItemMenu
     */

    private Node createItemMenu( String arg )
    {
        // {\\newmenu{itemMenu#1}{
        //    \\usemenu{createMenu}{Create Item}
        //    \\doclink{javascript:window.document.MenuTest.deleteItem("#1")}
        //      {Delete Item}
        //  }
        //  \\usemenu{itemMenu#1}{#1}}

        Document doc = getDocument();

        // Create the create usemenu text
        Element ctext = doc.createElement( "compound" );
        ctext.appendChild( createControlSequence( "_BEGINGROUP" ) );
        ctext.appendChild( doc.createTextNode( "Create Item" ) );
        ctext.appendChild( createControlSequence( "_ENDGROUP" ) );

        // Create the create usemenu command
        Element createMenu = doc.createElement( "menu" );
        createMenu.setAttribute( "menu-name", "createMenu" );
        setNth( createMenu, 0, ctext );

        // Create the menu item text
        Element mtext = doc.createElement( "compound" );
        mtext.appendChild( createControlSequence( "_BEGINGROUP" ) );
        mtext.appendChild( doc.createTextNode( "Delete Item" ) );
        mtext.appendChild( createControlSequence( "_ENDGROUP" ) );

        // Create the menu item command
        String jstext = "javascript:window.document.MenuTest." +
            "deleteItem('" + arg + "')";
        Element mcommand = doc.createElement( "compound" );
        mcommand.appendChild( doc.createTextNode( jstext ) );

        // Create the javascript link
        Element link = doc.createElement( "document-link" );
        setNth( link, 0, mtext );
        setNth( link, 1, mcommand );

        // Create the newmenu argument
        Element narg = doc.createElement( "compound" );
        narg.appendChild( createMenu );
        narg.appendChild( link );

        // Create the newmenu command
        Element newMenu = doc.createElement( "newmenu" );
        setNth( newMenu, 0, doc.createTextNode( "itemMenu" + arg ) );
        setNth( newMenu, 1, narg );

        // Create the usemenu command
        Element useMenu = doc.createElement( "menu" );
        useMenu.setAttribute( "menu-name", "itemMenu" + arg );
        setNth( useMenu, 0, createControlSequence( arg ) );

        // Create the outermost group
        Element group = doc.createElement( "group" );
        group.appendChild( newMenu );
        group.appendChild( useMenu );

        return group;
    }

    /*
     * createControlSequence
     */

    private Node createControlSequence( String name )
    {
        Element cs = getDocument().createElement( "control-sequence" );
        cs.setAttribute( "control-sequence", name );
        return cs;
    }

    /*
     * findElementByTagName
     */

    private Node findElementByTagName( String name )
    {
        Node doc = getDocument().getDocumentElement();
        return findElementByTagName( doc, name );
    }

    private static Node findElementByTagName( Node node, String name )
    {
        if ( node == null )
            return null;

        if ( name.equals( node.getNodeName() ) )
            return node;

        NodeList elts = node.getChildNodes();
        int eltc = elts.getLength();

        Node rnode = null;
        for ( int i = 0; i < eltc && rnode == null; i += 1 )
            rnode = findElementByTagName( elts.item(i), name );

        return rnode;
    }

    /*
     * findControlSequence
     */

    private Node findControlSequence( String name )
    {
        Element doc = getDocument().getDocumentElement();
        return findControlSequence( doc, name );
    }

    private static Node findControlSequence( Node node, String name )
    {
        if ( node == null )
            return null;

        if ( node.getNodeType() != Node.ELEMENT_NODE )
            return null;

        Element elt = (Element) node;

        if ( elt.getNodeName().equals( "control-sequence" ) )
            if ( elt.getAttribute( "control-sequence" ).equals( name ) )
                return node;

        NodeList elts = elt.getChildNodes();
        int eltc = elts.getLength();

        Node rnode = null;
        for ( int i = 0; i < eltc && rnode == null; i += 1 )
            rnode = findControlSequence( elts.item(i), name );

        return rnode;
    }

    /*
     * redraw
     */

    private void redraw()
    {
        if ( ownerDocument != null )
            ((TEDocument) ownerDocument).recomposeAndRedraw();
    }

    /*
     * isNodeName
     */

    public static boolean isNodeName( Node node, String name )
    {
        if ( node == null || name == null )
            return false;
        else
            return name.equals( node.getNodeName() );
    }

    /*
     * replaceNode
     */

    private static void replaceNode( Node newNode, Node oldNode )
    {
        Node parent = oldNode.getParentNode();
        if ( parent != null )
            parent.replaceChild( newNode, oldNode );
    }

    /*
     * setNth
     */

    public static void setNth( Node node, int n, Node elt )
    {
        node.replaceChild( elt, node.getChildNodes().item(n) );
    }
}

//////////////////////////////////////////////////////////////////////////////
