///////////////////////////////////////////////////////////////////////
//
// CircuitEdge: 
//
///////////////////////////////////////////////////////////////////////
//
// Copyright (c) 1998-99 by the IBM Corporation. All Rights Reserved.
//
// Author:          Barry M. Trager   bmt@watson.ibm.com
//                  Angel L. Diaz     aldiaz@us.ibm.com                  
//
// IBM MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
// THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
// TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
// PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IBM SHALL NOT BE LIABLE FOR
// ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING OR MODIFYING
// THIS SOFTWARE OR ITS DERIVATIVES.
//
///////////////////////////////////////////////////////////////////////

import java.awt.*;

public class CircuitEdge extends CLabelledLine
{
	public String actionString = "Draw Wire";

	CircuitEdge(Point p1, Point p2)
	{
		super(p1, p2);
	}

	CircuitEdge(Point p1, Point p2, String text)
	{
		super(p1, p2, "");
	}

	CircuitEdge(Line ln, String text)
	{
		super(ln, text);
	}

	CircuitEdge(CLabelledLine ln)
	{
		super(ln, "");
	}

	public Point getPoint(int num, int den)
	{
		int ex = end.x - start.x;
		int ey = end.y - start.y;

		return new Point(start.x + ex * num / den, start.y + ey * num / den);
	}

	/*
	 * new point is dx along line and dy perp to line from p0
	 */

	public Point getPoint(Point p0, int dx, int dy)
	{
		Point   p1 = start;
		Point   p2 = end;
		int	ex = p2.x - p1.x;
		int	ey = p2.y - p1.y;
		int	r = distance(p1, p2);

		// dx == desired pixels parallel to the edge
		// dy == desired pixels normal to the edge
		return new Point(p0.x + (dx * ex + dy * ey) / r, p0.y + (dx * ey - dy * ex) / r);
	}

	static public int distance(Point p1, Point p2)
	{
		int dx = p1.x - p2.x;
		int dy = p1.y - p2.y;
		return (int)Math.round(Math.sqrt(dx*dx + dy*dy));
	}

	static public void drawLine(Graphics g, Point p1, Point p2)
	{
		g.drawLine(p1.x, p1.y, p2.x, p2.y);
	}

	public void drawLabel(Graphics g, int xoffset, int yoffset)
	{
		if (label == null || label.length()==0) return;
		int x = end.x - start.x;
		int y = end.y - start.y;
		int xmid = start.x + x / 2;
		int ymid = start.y + y / 2;

		if (x*y < 0) 
		yoffset = - yoffset + g.getFontMetrics().getMaxAscent();
		g.drawString(label, xmid + xoffset, ymid + yoffset);
	}

	public void draw(Graphics g)
	{
		drawLine(g, start, end);
		drawLabel(g,5,-5);
	}

}

